#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <err.h>
#include "mohawk.h"

#ifndef DEFAULT_USER
#define DEFAULT_USER "nobody"
#endif /* DEFAULT_USER */

struct httpd conf;

void
dump_httpd()
{
	struct listener *listener;
	struct vhost *vh;
	struct mohawk_list *list;
	struct mohawk_hash *hash;
    struct addrinfo *ai;
	char hostname[NI_MAXHOST] = "";
	char service[NI_MAXSERV] = "";
	int error;

	printf("Dump httpd\n\
Debug: '%d'\n\
Mime file: '%s'\n\
Chroot: '%s'\n\
User: '%s'\n\
",
	conf.debug,
	conf.mimefile ? conf.mimefile : "",
	conf.chroot ? conf.chroot : "",
	conf.user
	);

	SLIST_FOREACH(listener, &conf.list, entry) {
		printf("Listen on ");
		ai = listener->ai;
        error = getnameinfo(ai->ai_addr, ai->ai_addrlen, hostname, NI_MAXHOST, service, NI_MAXSERV, NI_NUMERICHOST|NI_NUMERICSERV); 
        if (error != 0)
        {
            fprintf(stderr, "error in getnameinfo: %s\n", gai_strerror(error));
            continue;
        }
        if (*hostname != '\0')
            printf("%s ", hostname);
		if (*service != '\0')
            printf("port %s", service);
		printf("\n");
	}

	SLIST_FOREACH(vh, &conf.vhosts, entry) {
		printf("Vhost '%s'\n\
\tRootdir: '%s'\n\
\tCharset: '%s'\n\
\tDirlist css url: '%s'\n\
\tStatus url: '%s'\n\
\tMaxage: '%d'\n\
\tX_forwarded_for: '%d'\n\
",
		vh->host, vh->rootdir,
		vh->charset ? vh->charset : "",
		vh->dirlist_css_url ? vh->dirlist_css_url : "",
		vh->status_url ? vh->status_url : "",
		vh->maxage, vh->x_forwarded_for
		);
		printf("\tCGI pattern:\n");
		TAILQ_FOREACH(list, &vh->cgi_patterns, entry) {
			printf("\t\t'%s'\n", list->item);
		}
		printf("\tNo log pattern:\n");
		TAILQ_FOREACH(list, &vh->no_log_patterns, entry) {
			printf("\t\t'%s'\n", list->item);
		}
		printf("\tIndex name:\n");
		TAILQ_FOREACH(list, &vh->index_names, entry) {
			printf("\t\t'%s'\n", list->item);
		}
		printf("\tRestrict from:\n");
		TAILQ_FOREACH(list, &vh->restrict_froms, entry) {
			printf("\t\t'%s'\n", list->item);
		}
		printf("\tCGI map:\n");
		TAILQ_FOREACH(hash, &vh->cgi_map, entry) {
			printf("\t\t'%s' => '%s'\n", hash->key, hash->value);
		}
		printf("\tEnv:\n");
		TAILQ_FOREACH(hash, &vh->cgi_env, entry) {
			printf("\t\t'%s' => '%s'\n", hash->key, hash->value);
		}
	}
}

int
main(int argc, char **argv)
{
	/* configuration initialization */
	SLIST_INIT(&conf.list);
	SLIST_INIT(&conf.vhosts);

	conf.servername = NULL;
	conf.chroot = NULL;
	conf.mimefile = NULL;
	conf.mimelist = NULL;
	conf.debug = 0;
	conf.user = DEFAULT_USER;

	if (argc != 2)
		errx(1, "Usage: %s file.conf\n", argv[0]);

	parse_config(argv[1]);
	dump_httpd();
	return 0;
}

/* vim: set sw=4 sts=4 ts=4 : */
