/* mohawk - small HTTP server
**
** Copyright © 2010 by Baptiste Daroussin <bapt@FreeBSD.org>,
** Copyright © 2010 by Freddy Dissaux <freddy.dsx@free.fr>,
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
*/

#ifndef _MOHAWK_H
#define _MOHAWK_H

#include <time.h>
#include <stdarg.h>
#include <sys/queue.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/resource.h>
#include <sys/sbuf.h>
#include <netdb.h>

#include <libutil.h>

#define MOHAWK_VERSION "0.7"

#define CGI_READ 0
#define CGI_WRITE 1
#define CGI_ERR 2

#define CGI_HEADERS 0
#define CGI_BODY 2
#define CGI_DONE 3
#define CLIENT_READ 4
#define CLIENT_WRITE 5
#define CLIENT_SENDF 6
#define CLIENT_SENDFILE 7

struct listener {
	int fd;
	struct addrinfo *ai;
	SLIST_ENTRY(listener) entry;
};

struct mohawk_list {
	char *item;
	TAILQ_ENTRY(mohawk_list) entry;
};

struct mohawk_hash {
	char *key;
	char *value;
	TAILQ_ENTRY(mohawk_hash) entry;
};

struct vhost {
	TAILQ_HEAD(, mohawk_list) cgi_patterns;
	TAILQ_HEAD(, mohawk_list) no_log_patterns;
	TAILQ_HEAD(, mohawk_list) index_names;
	TAILQ_HEAD(, mohawk_list) restrict_froms;
	TAILQ_HEAD(, mohawk_hash) cgi_env;
	TAILQ_HEAD(, mohawk_hash) cgi_map;
	char *rootdir;
	char *host;
	char *dirlist_css_url;
	char *charset;
	char *status_url;
	int maxage;
	int x_forwarded_for;
	SLIST_ENTRY(vhost) entry;
};

struct httpd {
	SLIST_HEAD(, listener) list;
	SLIST_HEAD(, vhost) vhosts;
	char *chroot;
	char *user;
	char *mimefile;
	char *servername;
	char *logfile;
	char *pidfile;
	struct pidfh *pfh;
	int debug;
	properties mimelist;
};

struct client {
	int fd;
	char raw_header[8192]; /* limit to 8k as apache does */
	char cgi_header[2048];
	size_t header_len;
	size_t cgi_header_len;
	char remote_addr[NI_MAXHOST];
	struct sockaddr_storage ss;
	char server_port[NI_MAXSERV];
	int method;
	off_t offset;
	size_t nwrite;
	struct sbuf *response;
	char *script_name;
	char *query_string;
	char *remote_user;
	char *real_remote_addr;
	char *path_info;
	char *user_agent;
	char *referer;
	char *cookie;
	char *host;
	char *content_type;
	char *authorization;
	char *protocol;
	time_t if_modified_since;
	size_t content_length;
	int status;
	int state;
	int sendfile_fd;
	pid_t cgi_pid;
	int cgi_in_fd;
	int cgi_out_fd;
	int cgi_err_fd;
	struct sf_hdtr sf_hdtr;
	struct vhost *vhost;
};

extern struct httpd conf;

void parse_config(const char *);
int host(const char *host, const char *service);

#endif
/* vim: set sw=4 sts=4 ts=4 : */
